//+------------------------------------------------------------------+
//|                                                  ftsdPadawan.mq4 |
//|                                    Copyright  2007, ftsdPadawan |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, ftsdPadawan"
#property link      ""

//---- input parameters
extern double TakeProfit = 250.0;
extern double Lots = 0.1;
extern double TrailingStop = 35.0;

string orderTypes[] = {"BUY", "SELL", "BUYLIMIT", "BUYSTOP", "SELLLIMIT", "SELLSTOP"};

string magicNumberStrategies[] = {"Coders Guru Lesson 16 ~ modular", "Simple Moving Average 10/40"};
#define cg_l16    0
#define sma10_40  1

int cg_l16_isCrossed = 0;

int totalOrdersCountMax = 1;

void
applyEntryStrategy()
{
	entryStrategy_cg_l16();
	
	//Just to show my modular concept
	//entryStrategy_sma10_40();
}

void
applyExitStrategy()
{
   int orderIndex = 0;
   int totalOrdersCount = OrdersTotal();
	for(; orderIndex<totalOrdersCount; orderIndex++)
	{
		if(selectOrderByIndex(orderIndex))
		{
		   switch(OrderMagicNumber())
		   {
		      case cg_l16:
		         exitStrategy_cg_l16();
		         break;
		      case sma10_40:
	            //Just to show my modular concept
		         //exitStrategy_sma10_40();
		         break;
		      default:
		         break;
		   }
		}
	}
}

bool
canEnterMarket()
{
	if(OrdersTotal() >= totalOrdersCountMax)
	{
	   return (false);
	}
	
	if(isSane())
	{
		return (true);
	}
	else
	{
		return (false);
	}
}

int
cg_l16_Crossed (double line1 , double line2)
{
	static int last_direction = 0;
	static int current_direction = 0;
	if(line1>line2)current_direction = 1;    //up
	if(line1<line2)current_direction = 2;    //down
	if(current_direction != last_direction)  //changed
	{
		last_direction = current_direction;
		return (last_direction);
	}
	else
	{
		return (0);
	}
}
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
{
//----
   
//----
   return(0);
}

void
entryStrategy_cg_l16()
{
   int ticket;
	double shortEma = iMA(NULL,0,8,0,MODE_EMA,PRICE_CLOSE,0);
	double longEma = iMA(NULL,0,13,0,MODE_EMA,PRICE_CLOSE,0);
	cg_l16_isCrossed = cg_l16_Crossed(shortEma,longEma);

	if(OrdersTotal() < 1)
	{
		if(cg_l16_isCrossed == 1)
		{
			ticket=OrderSend(Symbol(),OP_BUY,Lots,Ask,3,0,Ask+TakeProfit*Point,magicNumberStrategies[cg_l16],cg_l16,0,Green);
			if(ticket>0)
			{
            selectOrderByTicket(ticket);
			}
		}
		if(cg_l16_isCrossed == 2)
		{
			ticket=OrderSend(Symbol(),OP_SELL,Lots,Bid,3,0,Bid-TakeProfit*Point,magicNumberStrategies[cg_l16],cg_l16,0,Red);
			if(ticket>0)
			{
            selectOrderByTicket(ticket);
			}
		}
	}
}

void
exitStrategy_cg_l16()
{
	double shortEma = iMA(NULL,0,8,0,MODE_EMA,PRICE_CLOSE,0);
	double longEma = iMA(NULL,0,13,0,MODE_EMA,PRICE_CLOSE,0);
	cg_l16_isCrossed = cg_l16_Crossed(shortEma,longEma);

   int orderIndex = 0;
   int totalOrdersCount = OrdersTotal();
	for(; orderIndex<totalOrdersCount; orderIndex++)
	{
		if(selectOrderByIndex(orderIndex))
		{
		   if(OrderType()<=OP_SELL && OrderSymbol()==Symbol() && OrderMagicNumber()==cg_l16)
		   {
			   if(OrderType()==OP_BUY) // long position is opened
			   {
			      // should it be closed?
				   if(cg_l16_isCrossed == 2)
				   {
					   OrderClose(OrderTicket(),OrderLots(),Bid,3,Violet);
					   // close position
					   return; // exit
				   }
				   // check for trailing stop
				   if(TrailingStop>0)
				   {
					   if(Bid-OrderOpenPrice()>Point*TrailingStop)
					   {
						   if(OrderStopLoss()<Bid-Point*TrailingStop)
						   {
							   OrderModify(OrderTicket(),OrderOpenPrice(),Bid-Point*TrailingStop,OrderTakeProfit(),0,Green);
							   return;
						   }
					   }
				   }
			   }
			   else // go to short position
			   {
				   // should it be closed?
				   if(cg_l16_isCrossed == 1)
				   {
					   OrderClose(OrderTicket(),OrderLots(),Ask,3,Violet);
					   // close position
					   return; // exit
				   }
				   // check for trailing stop
				   if(TrailingStop>0)
				   {
					   if((OrderOpenPrice()-Ask)>(Point*TrailingStop))
					   {
						   if((OrderStopLoss()>(Ask+Point*TrailingStop)) || (OrderStopLoss()==0))
						   {
							   OrderModify(OrderTicket(),OrderOpenPrice(),Ask+Point*TrailingStop,OrderTakeProfit(),0,Red);
							   return;
						   }
					   }
				   }
			   }
			}
		}
	}
}

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
//----
   
//----
   return(0);
}

bool
isSane()
{
	if(Bars<100)
	{
		Print("Bars less than 100");
		return(false);
	}
	if(TakeProfit<10)
	{
		Print("TakeProfit less than 10.  Check TakeProfit!");
		return(false);
	}
	return (true);
}

bool
selectOrderByIndex(int index)
{
   if(OrderSelect(index, SELECT_BY_POS, MODE_TRADES))
   {
      Print("Selected ", orderTypes[OrderType()], " order using strategy: ", magicNumberStrategies[OrderMagicNumber()]);
      return (true);
   }
   else
   {
      Print("Error selecting order: ", GetLastError());
      return (false);
   }
}

bool
selectOrderByTicket(int ticket)
{
   if(OrderSelect(ticket, SELECT_BY_TICKET, MODE_TRADES))
   {
      Print(orderTypes[OrderType()], " order opened at ", OrderOpenPrice(), " using strategy: ", magicNumberStrategies[OrderMagicNumber()]);
      return (true);
   }
   else
   {
		Print("Error opening ", orderTypes[OrderType()], " order: ", GetLastError());
      return (false);
   }
}

//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int
start()
{
	if(canEnterMarket())
	{
		applyEntryStrategy();
	}

	if(OrdersTotal() > 0)
	{
	   applyExitStrategy();
	}
	
	return (0);
}

